/* cacheflush.h: cache flushing
 *
 * Copyright (C) 2005 Red Hat, Inc. All Rights Reserved.
 * Written by David Howells (dhowells@redhat.com)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 */
#ifndef _ASM_CACHEFLUSH_H
#define _ASM_CACHEFLUSH_H

/* Keep includes the same across arches.  */
#include <linux/sched.h>
#include <linux/errno.h>

/*
 * virtually-indexed cache managment (our cache is physically indexed)
 */
#define flush_cache_all()			do {} while(0)
#define flush_cache_mm(mm)			do {} while(0)
#define flush_cache_range(mm, start, end)	do {} while(0)
#define flush_cache_page(vma, vmaddr)		do {} while(0)
#define flush_cache_vmap(start, end)		do {} while(0)
#define flush_cache_vunmap(start, end)		do {} while(0)
#define flush_dcache_mmap_lock(mapping)		do {} while(0)
#define flush_dcache_mmap_unlock(mapping)	do {} while(0)

/*
 * physically-indexed cache managment
 */
#ifndef CONFIG_MN10300_NOCPUCACHING

#define flush_page_to_ram(page) \
	mn10300_dcache_flush_page(page_to_phys(page))

extern void flush_dcache_page(struct page *page);
extern void flush_icache_range(unsigned long start, unsigned long end);
extern void flush_icache_page(struct vm_area_struct *vma, struct page *pg);

#else

#define flush_page_to_ram(page)			do { } while(0)
#define flush_dcache_page(page)			do { } while(0)
#define flush_icache_range(start,end)		do { } while(0)
#define flush_icache_page(vma,pg)		do { } while(0)

#endif

#define flush_icache_user_range(vma,pg,adr,len) \
	flush_icache_range(adr, adr + len - 1)

#define copy_to_user_page(vma, page, vaddr, dst, src, len) \
	memcpy(dst, src, len)

#define copy_from_user_page(vma, page, vaddr, dst, src, len) \
	memcpy(dst, src, len)

#ifdef CONFIG_DEBUG_PAGEALLOC
/* internal debugging function */
extern void kernel_map_pages(struct page *page, int numpages, int enable);
#endif

#endif /* _ASM_CACHEFLUSH_H */
