/* frame.inc: MN10300 Microcontroller core system registers -*- asm -*-
 *
 * Copyright (C) 2005 Red Hat, Inc. All Rights Reserved.
 * Written by David Howells (dhowells@redhat.com)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 */

#ifndef _ASM_FRAME_INC
#define _ASM_FRAME_INC

#ifndef __ASSEMBLY__
#error not for use in C files
#endif

#ifndef __ASM_OFFSETS_H__
#include <asm/asm_offsets.h>
#endif

#define pi break

#define fp a3

###############################################################################
#
# build a stack frame from the registers
# - the caller has subtracted 4 from SP before coming here
#
###############################################################################
.macro SAVE_ALL
	add	-4,sp				# next exception frame ptr save area
	movm	[other],(sp)
	movm	[d2,d3,a2,a3,exreg0,exreg1,exother],(sp)
	mov	sp,fp				# FRAME pointer in A3
	add	-12,sp				# allow for calls to be made
	mov	usp,a1
        mov	a1,(REG_SP,fp)
	mov	(__frame),a1
        mov	a1,(REG_NEXT,fp)
	mov	fp,(__frame)

	and	~EPSW_FE,epsw			# disable the FPU inside the kernel
.endm

###############################################################################
#
# restore the registers from a stack frame
#
###############################################################################
.macro RESTORE_ALL
	# peel back the stack to the calling frame
	# - this permits execve() to discard extra frames due to kernel syscalls
	mov	(__frame),fp
	mov	fp,sp
        mov	(REG_NEXT,fp),d0
	mov	d0,(__frame)
	or	d0,d0
	bne	999f				# userspace has regs->next == 0

	# return to userspace
	movm	(sp),[d2,d3,a2,a3,exreg0,exreg1,exother]
	mov	(sp),a1
	mov	a1,usp

	btst	EPSW_T,d0
	beq	99f

20:	or	EPSW_NMID,epsw
	movhu	(DCR),d1
	or	0x0001, d1
	movhu	d1,(DCR)

99:	movm	(sp),[other]
	add	8,sp
	rti

	# return to kernelspace
999:	movm	(sp),[d2,d3,a2,a3,exreg0,exreg1,exother]
	movm	(sp),[other]
	add	8,sp
	rti

.endm


#endif /* _ASM_FRAME_INC */
